@@ -23,7 +23,9 @@
 #  \code{length(dim)} columns.
 # }
 #
-# @examples "../incl/arrayIndex.Rex"
+# \examples{\donttest{
+# @include "../incl/arrayIndex.Rex"
+# }}
 #
 # \references{
 #  [1] H. Bengtsson, \emph{Bayesian Networks - a self-contained 
@@ -40,31 +42,8 @@
 #
 # @keyword manip
 # @keyword utilities
+# @keyword internal
 #*/########################################################################### 
 setMethodS3("arrayIndex", "default", function(i, dim, ...) {
-  ndim <- length(dim);       # number of dimension
-  dim <- as.integer(dim);
-  pi <- cumprod(c(1L,dim));  # base
-
-  # Allocate return matrix
-  naValue <- as.integer(NA);
-  j <- matrix(naValue, nrow=length(i), ncol=ndim);
-
-  i <- (i-1);
-  for (kk in 1:ndim) {
-    j[,kk] <- as.integer(floor((i %% pi[kk+1])/pi[kk]));
-  }
-  j <- j + 1L;
-
-  j;
+  .Defunct(msg = "R.utils::arrayIndex() is defunct. Instead, use base::arrayInd().")
 })
-
-############################################################################
-# HISTORY:
-# 2010-07-05
-# o Now arrayIndex() returns an @integer @matrix.
-# o Added an Rd link to base::arrayInd.
-# 2006-03-07
-# o Created as an answer to R-help thread "[R] returning the largest element
-#   in an array/matrix?" on 2006-03-07.
-############################################################################
